@@ -65,7 +65,7 @@ from streamlit.elements.pyplot import PyplotMixin
 from streamlit.elements.write import WriteMixin
 from streamlit.elements.layouts import LayoutsMixin
 from streamlit.elements.form import FormMixin, FormData, current_form_id
-from streamlit.widgets import NoValue
+from streamlit.state.widgets import NoValue
 
 LOGGER = get_logger(__name__)
 
@@ -340,9 +340,11 @@ class DeltaGenerator(
 
         Returns
         -------
-        DeltaGenerator
-            A DeltaGenerator that can be used to modify the newly-created
-            element.
+        DeltaGenerator or any
+            If this element is NOT an interactive widget, return a
+            DeltaGenerator that can be used to modify the newly-created
+            element. Otherwise, if the element IS a widget, return the
+            `return_value` parameter.
 
         """
         # Operate on the active DeltaGenerator, in case we're in a `with` block.
