@@ -17,8 +17,15 @@ from typing import Optional, cast
 import streamlit
 from streamlit.errors import StreamlitAPIException
 from streamlit.proto.Button_pb2 import Button as ButtonProto
-from streamlit.widgets import register_widget
+from streamlit.state.session_state import (
+    WidgetArgs,
+    WidgetCallback,
+    WidgetDeserializer,
+    WidgetKwargs,
+)
+from streamlit.state.widgets import register_widget
 from .form import current_form_id, is_in_form
+from .utils import check_callback_rules, check_session_state_rules
 
 
 FORM_DOCS_INFO = """
@@ -29,7 +36,15 @@ For more information, refer to the
 
 
 class ButtonMixin:
-    def button(self, label, key=None, help=None):
+    def button(
+        self,
+        label,
+        key=None,
+        help=None,
+        on_click=None,
+        args=None,
+        kwargs=None,
+    ) -> bool:
         """Display a button widget.
 
         Parameters
@@ -42,7 +57,14 @@ class ButtonMixin:
             based on its content. Multiple widgets of the same type may
             not share the same key.
         help : str
-            A tooltip that gets displayed when the button is hovered over.
+            An optional tooltip that gets displayed when the button is
+            hovered over.
+        on_click : callable
+            An optional callback invoked when this button is clicked.
+        args : tuple
+            An optional tuple of args to pass to the callback.
+        kwargs : dict
+            An optional dict of kwargs to pass to the callback.
 
         Returns
         -------
@@ -57,7 +79,15 @@ class ButtonMixin:
         ...     st.write('Goodbye')
 
         """
-        return self.dg._button(label, key, help, is_form_submitter=False)
+        return self.dg._button(
+            label,
+            key,
+            help,
+            is_form_submitter=False,
+            on_click=on_click,
+            args=args,
+            kwargs=kwargs,
+        )
 
     def _button(
         self,
@@ -65,8 +95,13 @@ class ButtonMixin:
         key: Optional[str],
         help: Optional[str],
         is_form_submitter: bool,
-    ) -> "streamlit.delta_generator.DeltaGenerator":
-        button_proto = ButtonProto()
+        on_click: Optional[WidgetCallback] = None,
+        args: Optional[WidgetArgs] = None,
+        kwargs: Optional[WidgetKwargs] = None,
+    ) -> bool:
+        if not is_form_submitter:
+            check_callback_rules(self.dg, on_click)
+        check_session_state_rules(default_value=None, key=key, writes_allowed=False)
 
         # It doesn't make sense to create a button inside a form (except
         # for the "Form Submitter" button that's automatically created in
@@ -80,9 +115,10 @@ class ButtonMixin:
                 )
             elif not is_in_form(self.dg) and is_form_submitter:
                 raise StreamlitAPIException(
-                    f"`st.submit_button()` must be used inside an `st.form()`.{FORM_DOCS_INFO}"
+                    f"`st.form_submit_button()` must be used inside an `st.form()`.{FORM_DOCS_INFO}"
                 )
 
+        button_proto = ButtonProto()
         button_proto.label = label
         button_proto.default = False
         button_proto.is_form_submitter = is_form_submitter
@@ -90,10 +126,21 @@ class ButtonMixin:
         if help is not None:
             button_proto.help = help
 
-        ui_value = register_widget("button", button_proto, user_key=key)
-        current_value = ui_value if ui_value is not None else False
+        def deserialize_button(ui_value, widget_id=""):
+            return ui_value or False
 
-        return self.dg._enqueue("button", button_proto, current_value)  # type: ignore
+        current_value, _ = register_widget(
+            "button",
+            button_proto,
+            user_key=key,
+            on_change_handler=on_click,
+            args=args,
+            kwargs=kwargs,
+            deserializer=deserialize_button,
+            serializer=bool,
+        )
+        self.dg._enqueue("button", button_proto)
+        return cast(bool, current_value)
 
     @property
     def dg(self) -> "streamlit.delta_generator.DeltaGenerator":
