@@ -19,8 +19,9 @@ import streamlit
 from streamlit.errors import StreamlitAPIException
 from streamlit.js_number import JSNumber, JSNumberBoundsException
 from streamlit.proto.NumberInput_pb2 import NumberInput as NumberInputProto
-from streamlit.widgets import register_widget, NoValue
+from streamlit.state.widgets import register_widget, NoValue
 from .form import current_form_id
+from .utils import check_callback_rules, check_session_state_rules
 
 
 class NumberInputMixin:
@@ -34,6 +35,9 @@ class NumberInputMixin:
         format=None,
         key=None,
         help=None,
+        on_change=None,
+        args=None,
+        kwargs=None,
     ):
         """Display a numeric input widget.
 
@@ -64,7 +68,13 @@ class NumberInputMixin:
             based on its content. Multiple widgets of the same type may
             not share the same key.
         help : str
-            A tooltip that gets displayed next to the input.
+            An optional tooltip that gets displayed next to the input.
+        on_change : callable
+            An optional callback invoked when this number_input's value changes.
+        args : tuple
+            An optional tuple of args to pass to the callback.
+        kwargs : dict
+            An optional dict of kwargs to pass to the callback.
 
         Returns
         -------
@@ -77,15 +87,20 @@ class NumberInputMixin:
         >>> number = st.number_input('Insert a number')
         >>> st.write('The current number is ', number)
         """
+        check_callback_rules(self.dg, on_change)
+        check_session_state_rules(default_value=value, key=key)
 
         # Ensure that all arguments are of the same type.
-        args = [min_value, max_value, value, step]
+        number_input_args = [min_value, max_value, value, step]
 
         int_args = all(
-            isinstance(a, (numbers.Integral, type(None), NoValue)) for a in args
+            isinstance(a, (numbers.Integral, type(None), NoValue))
+            for a in number_input_args
         )
 
-        float_args = all(isinstance(a, (float, type(None), NoValue)) for a in args)
+        float_args = all(
+            isinstance(a, (float, type(None), NoValue)) for a in number_input_args
+        )
 
         if not int_args and not float_args:
             raise StreamlitAPIException(
@@ -201,10 +216,26 @@ class NumberInputMixin:
         if format is not None:
             number_input_proto.format = format
 
-        ui_value = register_widget("number_input", number_input_proto, user_key=key)
+        def deserialize_number_input(ui_value, widget_id=""):
+            return ui_value if ui_value is not None else value
 
-        return_value = ui_value if ui_value is not None else value
-        return self.dg._enqueue("number_input", number_input_proto, return_value)
+        current_value, set_frontend_value = register_widget(
+            "number_input",
+            number_input_proto,
+            user_key=key,
+            on_change_handler=on_change,
+            args=args,
+            kwargs=kwargs,
+            deserializer=deserialize_number_input,
+            serializer=lambda x: x,
+        )
+
+        if set_frontend_value:
+            number_input_proto.value = current_value
+            number_input_proto.set_value = True
+
+        self.dg._enqueue("number_input", number_input_proto)
+        return current_value
 
     @property
     def dg(self) -> "streamlit.delta_generator.DeltaGenerator":
