@@ -17,20 +17,33 @@ from typing import cast
 import streamlit
 from streamlit.errors import StreamlitAPIException
 from streamlit.proto.Selectbox_pb2 import Selectbox as SelectboxProto
-from streamlit.type_util import ensure_iterable
-from streamlit.widgets import register_widget, NoValue
+from streamlit.state.widgets import register_widget, NoValue
+from streamlit.type_util import OptionSequence, ensure_indexable
+from streamlit.util import index_
 from .form import current_form_id
+from .utils import check_callback_rules, check_session_state_rules
 
 
 class SelectboxMixin:
-    def selectbox(self, label, options, index=0, format_func=str, key=None, help=None):
+    def selectbox(
+        self,
+        label,
+        options: OptionSequence,
+        index=0,
+        format_func=str,
+        key=None,
+        help=None,
+        on_change=None,
+        args=None,
+        kwargs=None,
+    ):
         """Display a select widget.
 
         Parameters
         ----------
         label : str
             A short label explaining to the user what this select widget is for.
-        options : list, tuple, numpy.ndarray, pandas.Series, or pandas.DataFrame
+        options : Sequence, numpy.ndarray, pandas.Series, pandas.DataFrame, or pandas.Index
             Labels for the select options. This will be cast to str internally
             by default. For pandas.DataFrame, the first column is selected.
         index : int
@@ -44,7 +57,13 @@ class SelectboxMixin:
             based on its content. Multiple widgets of the same type may
             not share the same key.
         help : str
-            A tooltip that gets displayed next to the selectbox.
+            An optional tooltip that gets displayed next to the selectbox.
+        on_change : callable
+            An optional callback invoked when this selectbox's value changes.
+        args : tuple
+            An optional tuple of args to pass to the callback.
+        kwargs : dict
+            An optional dict of kwargs to pass to the callback.
 
         Returns
         -------
@@ -60,14 +79,17 @@ class SelectboxMixin:
         >>> st.write('You selected:', option)
 
         """
-        options = ensure_iterable(options)
+        check_callback_rules(self.dg, on_change)
+        check_session_state_rules(default_value=None if index == 0 else index, key=key)
+
+        opt = ensure_indexable(options)
 
         if not isinstance(index, int):
             raise StreamlitAPIException(
                 "Selectbox Value has invalid type: %s" % type(index).__name__
             )
 
-        if len(options) > 0 and not 0 <= index < len(options):
+        if len(opt) > 0 and not 0 <= index < len(opt):
             raise StreamlitAPIException(
                 "Selectbox index must be between 0 and length of options"
             )
@@ -75,20 +97,38 @@ class SelectboxMixin:
         selectbox_proto = SelectboxProto()
         selectbox_proto.label = label
         selectbox_proto.default = index
-        selectbox_proto.options[:] = [str(format_func(option)) for option in options]
+        selectbox_proto.options[:] = [str(format_func(option)) for option in opt]
         selectbox_proto.form_id = current_form_id(self.dg)
         if help is not None:
             selectbox_proto.help = help
 
-        ui_value = register_widget("selectbox", selectbox_proto, user_key=key)
-        current_value = ui_value if ui_value is not None else index
+        def deserialize_select_box(ui_value, widget_id=""):
+            idx = ui_value if ui_value is not None else index
 
-        return_value = (
-            options[current_value]
-            if len(options) > 0 and options[current_value] is not None
-            else NoValue
+            return opt[idx] if len(opt) > 0 and opt[idx] is not None else None
+
+        def serialize_select_box(v):
+            if len(opt) == 0:
+                return 0
+            return index_(opt, v)
+
+        current_value, set_frontend_value = register_widget(
+            "selectbox",
+            selectbox_proto,
+            user_key=key,
+            on_change_handler=on_change,
+            args=args,
+            kwargs=kwargs,
+            deserializer=deserialize_select_box,
+            serializer=serialize_select_box,
         )
-        return self.dg._enqueue("selectbox", selectbox_proto, return_value)
+
+        if set_frontend_value:
+            selectbox_proto.value = serialize_select_box(current_value)
+            selectbox_proto.set_value = True
+
+        self.dg._enqueue("selectbox", selectbox_proto)
+        return current_value
 
     @property
     def dg(self) -> "streamlit.delta_generator.DeltaGenerator":
