@@ -18,13 +18,24 @@ import streamlit
 from streamlit.errors import StreamlitAPIException
 from streamlit.proto.TextArea_pb2 import TextArea as TextAreaProto
 from streamlit.proto.TextInput_pb2 import TextInput as TextInputProto
-from streamlit.widgets import register_widget
+from streamlit.state.widgets import register_widget
 from .form import current_form_id
+from .utils import check_callback_rules, check_session_state_rules
 
 
 class TextWidgetsMixin:
     def text_input(
-        self, label, value="", max_chars=None, key=None, type="default", help=None
+        self,
+        label,
+        value="",
+        max_chars=None,
+        key=None,
+        type="default",
+        help=None,
+        autocomplete=None,
+        on_change=None,
+        args=None,
+        kwargs=None,
     ):
         """Display a single-line text input widget.
 
@@ -47,7 +58,18 @@ class TextWidgetsMixin:
             a regular text input), or "password" (for a text input that
             masks the user's typed value). Defaults to "default".
         help : str
-            A tooltip that gets displayed next to the input.
+            An optional tooltip that gets displayed next to the input.
+        autocomplete : str
+            An optional value that will be passed to the <input> element's
+            autocomplete property. If unspecified, this value will be set to
+            "new-password" for "password" inputs, and the empty string for
+            "default" inputs. For more details, see https://developer.mozilla.org/en-US/docs/Web/HTML/Attributes/autocomplete
+        on_change : callable
+            An optional callback invoked when this text_input's value changes.
+        args : tuple
+            An optional tuple of args to pass to the callback.
+        kwargs : dict
+            An optional dict of kwargs to pass to the callback.
 
         Returns
         -------
@@ -60,6 +82,9 @@ class TextWidgetsMixin:
         >>> st.write('The current movie title is', title)
 
         """
+        check_callback_rules(self.dg, on_change)
+        check_session_state_rules(default_value=None if value == "" else value, key=key)
+
         text_input_proto = TextInputProto()
         text_input_proto.label = label
         text_input_proto.default = str(value)
@@ -80,12 +105,44 @@ class TextWidgetsMixin:
                 % type
             )
 
-        ui_value = register_widget("text_input", text_input_proto, user_key=key)
-        current_value = ui_value if ui_value is not None else value
-        return self.dg._enqueue("text_input", text_input_proto, str(current_value))
+        # Marshall the autocomplete param. If unspecified, this will be
+        # set to "new-password" for password inputs.
+        if autocomplete is None:
+            autocomplete = "new-password" if type == "password" else ""
+        text_input_proto.autocomplete = autocomplete
+
+        def deserialize_text_input(ui_value, widget_id="") -> str:
+            return str(ui_value if ui_value is not None else value)
+
+        current_value, set_frontend_value = register_widget(
+            "text_input",
+            text_input_proto,
+            user_key=key,
+            on_change_handler=on_change,
+            args=args,
+            kwargs=kwargs,
+            deserializer=deserialize_text_input,
+            serializer=lambda x: x,
+        )
+
+        if set_frontend_value:
+            text_input_proto.value = current_value
+            text_input_proto.set_value = True
+
+        self.dg._enqueue("text_input", text_input_proto)
+        return current_value
 
     def text_area(
-        self, label, value="", height=None, max_chars=None, key=None, help=None
+        self,
+        label,
+        value="",
+        height=None,
+        max_chars=None,
+        key=None,
+        help=None,
+        on_change=None,
+        args=None,
+        kwargs=None,
     ):
         """Display a multi-line text input widget.
 
@@ -107,7 +164,13 @@ class TextWidgetsMixin:
             based on its content. Multiple widgets of the same type may
             not share the same key.
         help : str
-            A tooltip that gets displayed next to the textarea.
+            An optional tooltip that gets displayed next to the textarea.
+        on_change : callable
+            An optional callback invoked when this text_area's value changes.
+        args : tuple
+            An optional tuple of args to pass to the callback.
+        kwargs : dict
+            An optional dict of kwargs to pass to the callback.
 
         Returns
         -------
@@ -126,6 +189,9 @@ class TextWidgetsMixin:
         >>> st.write('Sentiment:', run_sentiment_analysis(txt))
 
         """
+        check_callback_rules(self.dg, on_change)
+        check_session_state_rules(default_value=None if value == "" else value, key=key)
+
         text_area_proto = TextAreaProto()
         text_area_proto.label = label
         text_area_proto.default = str(value)
@@ -139,9 +205,26 @@ class TextWidgetsMixin:
         if max_chars is not None:
             text_area_proto.max_chars = max_chars
 
-        ui_value = register_widget("text_area", text_area_proto, user_key=key)
-        current_value = ui_value if ui_value is not None else value
-        return self.dg._enqueue("text_area", text_area_proto, str(current_value))
+        def deserialize_text_area(ui_value, widget_id="") -> str:
+            return str(ui_value if ui_value is not None else value)
+
+        current_value, set_frontend_value = register_widget(
+            "text_area",
+            text_area_proto,
+            user_key=key,
+            on_change_handler=on_change,
+            args=args,
+            kwargs=kwargs,
+            deserializer=deserialize_text_area,
+            serializer=lambda x: x,
+        )
+
+        if set_frontend_value:
+            text_area_proto.value = current_value
+            text_area_proto.set_value = True
+
+        self.dg._enqueue("text_area", text_area_proto)
+        return current_value
 
     @property
     def dg(self) -> "streamlit.delta_generator.DeltaGenerator":
