@@ -23,6 +23,7 @@ import threading
 from streamlit.proto.ForwardMsg_pb2 import ForwardMsg
 
 from streamlit.logger import get_logger
+from streamlit import util
 
 LOGGER = get_logger(__name__)
 
@@ -41,6 +42,9 @@ class ReportQueue(object):
             # where delta_path = (container, parent block path as a string)
             self._delta_index_map = dict()
 
+    def __repr__(self) -> str:
+        return util.repr_(self)
+
     def get_debug(self):
         from google.protobuf.json_format import MessageToDict
 
