@@ -15,16 +15,16 @@
 import sys
 import uuid
 from enum import Enum
+from typing import Optional, TYPE_CHECKING
 
 import tornado.gen
 import tornado.ioloop
 
-import streamlit.elements.exception as exception
+import streamlit.elements.exception as exception_utils
 from streamlit import __version__
 from streamlit import caching
 from streamlit import config
 from streamlit import url_util
-from streamlit import util
 from streamlit.case_converters import to_snake_case
 from streamlit.credentials import Credentials
 from streamlit.logger import get_logger
@@ -46,6 +46,8 @@ from streamlit.uploaded_file_manager import UploadedFileManager
 from streamlit.watcher.local_sources_watcher import LocalSourcesWatcher
 
 LOGGER = get_logger(__name__)
+if TYPE_CHECKING:
+    from streamlit.state.session_state import SessionState
 
 
 class ReportSessionState(Enum):
@@ -113,6 +115,11 @@ class ReportSession(object):
 
         self._scriptrunner = None
 
+        # This needs to be lazily imported to avoid a dependency cycle.
+        from streamlit.state.session_state import SessionState
+
+        self._session_state = SessionState()
+
         LOGGER.debug("ReportSession initialized (id=%s)", self.id)
 
     def flush_browser_queue(self):
@@ -202,7 +209,7 @@ class ReportSession(object):
         self._on_scriptrunner_event(ScriptRunnerEvent.SCRIPT_STOPPED_WITH_SUCCESS)
 
         msg = ForwardMsg()
-        exception.marshall(msg.delta.new_element.exception, e)
+        exception_utils.marshall(msg.delta.new_element.exception, e)
 
         self.enqueue(msg)
 
@@ -229,6 +236,10 @@ class ReportSession(object):
         self._enqueue_script_request(ScriptRequest.RERUN, rerun_data)
         self._set_page_config_allowed = True
 
+    @property
+    def session_state(self) -> "SessionState":
+        return self._session_state
+
     def _on_source_file_changed(self):
         """One of our source files changed. Schedule a rerun if appropriate."""
         if self._run_on_save:
@@ -304,9 +315,6 @@ class ReportSession(object):
                     self._local_sources_watcher.update_watched_modules
                 )
             else:
-                # When a script fails to compile, we send along the exception.
-                import streamlit.elements.exception as exception_utils
-
                 msg = ForwardMsg()
                 exception_utils.marshall(
                     msg.session_event.script_compilation_exception, exception
@@ -488,6 +496,8 @@ class ReportSession(object):
         # terminal.
         caching.clear_cache()
 
+        self._session_state.clear_state()
+
     def handle_set_run_on_save_request(self, new_value):
         """Change our run_on_save flag to the given value.
 
@@ -548,6 +558,7 @@ class ReportSession(object):
             enqueue_forward_msg=self.enqueue,
             client_state=self._client_state,
             request_queue=self._script_request_queue,
+            session_state=self._session_state,
             uploaded_file_mgr=self._uploaded_file_mgr,
         )
         self._scriptrunner.on_event.connect(self._on_scriptrunner_event)
